"""Test helpers."""
from youtubeaio.models import YouTubeThumbnail


def get_thumbnail(resolution: str) -> YouTubeThumbnail:
    """Return mock thumbnail with resolution as url."""
    return YouTubeThumbnail(
        url=resolution,
        width=100,
        height=100,
    )



"""Tests for the YouTube client."""
import asyncio

import aiohttp
import pytest
from aiohttp.web_request import BaseRequest
from aresponses import Response, ResponsesMockServer

from youtubeaio.types import (
    ForbiddenError,
    UnauthorizedError,
    YouTubeAPIError,
    YouTubeBackendError,
    YouTubeResourceNotFoundError,
)
from youtubeaio.youtube import YouTube

from . import load_fixture
from .const import YOUTUBE_URL


async def test_new_session(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving a video."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("video_response_snippet.json"),
        ),
    )
    async with YouTube() as youtube:
        assert not youtube.session
        await youtube.get_video(video_id="Ks-_Mh1QhMc")
        assert youtube.session


async def test_timeout(aresponses: ResponsesMockServer) -> None:
    """Test request timeout."""

    # Faking a timeout by sleeping
    async def response_handler(_: BaseRequest) -> Response:
        """Response handler for this test."""
        await asyncio.sleep(2)
        return aresponses.Response(body="Goodmorning!")

    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        response_handler,
    )

    youtube = YouTube(session_timeout=1)
    with pytest.raises(YouTubeAPIError):
        assert await youtube.get_video(video_id="Ks-_Mh1QhMc")
    await youtube.close()


async def test_fetch_video_not_found(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving a non-existent video."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=404,
            headers={"Content-Type": "application/json"},
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(YouTubeResourceNotFoundError):
            await youtube.get_video(video_id="Ks-_Mh1QhMc")
        await youtube.close()


async def test_general_error_handling(
    aresponses: ResponsesMockServer,
) -> None:
    """Test throwing unexpected error."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=418,
            headers={"Content-Type": "application/json"},
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(YouTubeAPIError):
            await youtube.get_video(video_id="Ks-_Mh1QhMc")
        await youtube.close()


async def test_unexpected_server_response(
    aresponses: ResponsesMockServer,
) -> None:
    """Test handling a server error."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "plain/text"},
            text="Yes",
        ),
    )

    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(YouTubeAPIError):
            await youtube.get_video(video_id="Ks-_Mh1QhMc")
        await youtube.close()


async def test_internal_server_error(
    aresponses: ResponsesMockServer,
) -> None:
    """Test handling an internal server error."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=500,
            headers={"Content-Type": "plain/text"},
            text="Yes",
        ),
    )

    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(YouTubeBackendError):
            await youtube.get_video(video_id="Ks-_Mh1QhMc")
        await youtube.close()


async def test_bad_request(
    aresponses: ResponsesMockServer,
) -> None:
    """Test handling a bad request."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=400,
            headers={"Content-Type": "application/json"},
            text='{"message":"Something went wrong"}',
        ),
    )

    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(YouTubeAPIError):
            await youtube.get_video(video_id="Ks-_Mh1QhMc")
        await youtube.close()


async def test_unauthorized(
    aresponses: ResponsesMockServer,
) -> None:
    """Test handling being unauthorized."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=401,
            headers={"Content-Type": "application/json"},
            text='{"message":"Something went wrong"}',
        ),
    )

    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(UnauthorizedError):
            await youtube.get_video(video_id="Ks-_Mh1QhMc")
        await youtube.close()


async def test_not_activated(
    aresponses: ResponsesMockServer,
) -> None:
    """Test handling YouTube api not activated."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=403,
            headers={"Content-Type": "application/json"},
            text=load_fixture("youtube_not_activated.json"),
        ),
    )

    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(ForbiddenError):
            await youtube.get_video(video_id="Ks-_Mh1QhMc")
        await youtube.close()



"""Tests for the YouTube Library."""
import json
from pathlib import Path
from typing import Any


def load_fixture(filename: str) -> str:
    """Load a fixture."""
    path = Path(__package__) / "fixtures" / filename
    return path.read_text(encoding="utf-8")


def construct_fixture(object_type: str, parts: list[str], object_number: int) -> Any:
    """Construct a fixture from different files."""
    base_path = Path(__package__) / "fixtures" / object_type
    base_json = json.loads((base_path / "base.json").read_text(encoding="utf-8"))

    object_json = base_path / str(object_number)
    base_object_json = json.loads(
        (object_json / "base.json").read_text(encoding="utf-8"),
    )
    for part in parts:
        part_json_path = object_json / f"{part}.json"
        part_json = json.loads(part_json_path.read_text(encoding="utf-8"))
        base_object_json[part] = part_json
    base_json["items"].append(base_object_json)
    return base_json



"""Tests for the helper module."""
from collections.abc import AsyncGenerator
from typing import Any

import pytest

from youtubeaio.helper import build_scope, build_url, chunk, first, limit
from youtubeaio.types import AuthScope


async def _generator(amount: int) -> AsyncGenerator[int, None]:
    for i in range(0, amount):
        yield i


async def test_first() -> None:
    """Test if the first method works."""
    first_variable = await first(_generator(1))
    assert first_variable == 0


async def test_first_unavailable() -> None:
    """Test if the first method works."""
    second_variable = await first(_generator(0))
    assert second_variable is None


def test_chunk() -> None:
    """Test if the chunk method works."""
    source = list(range(0, 10))

    result = list(chunk(source, 3))
    assert result == [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]


async def test_limit() -> None:
    """Test if the limit method works."""
    async for i in limit(_generator(10), 3):
        assert i < 3
    async for i in limit(_generator(2), 3):
        assert i < 3


async def test_limit_invalid_value() -> None:
    """Test if the limit method works."""
    with pytest.raises(ValueError):
        await limit(_generator(10), 0).__anext__()


async def test_build_scope() -> None:
    """Test build scope."""
    assert (
        build_scope([AuthScope.READ_ONLY, AuthScope.MANAGE])
        == "https://www.googleapis.com/auth/youtube.readonly https://www.googleapis.com/auth/youtube"
    )


@pytest.mark.parametrize(
    ("params", "remove_none", "split_lists", "enum_value", "result"),
    [
        (
            {
                "hello": None,
            },
            True,
            True,
            True,
            "asd.com",
        ),
        (
            {
                "hello": None,
            },
            False,
            True,
            True,
            "asd.com?hello",
        ),
        (
            {
                "hello": [
                    "yes",
                    "no",
                ],
            },
            False,
            True,
            True,
            "asd.com?hello=yes&hello=no",
        ),
        (
            {
                "hello": [
                    "yes",
                    "no",
                ],
            },
            False,
            False,
            True,
            "asd.com?hello=%5B%27yes%27%2C%20%27no%27%5D",
        ),
        (
            {
                "hello": AuthScope.MANAGE,
            },
            False,
            True,
            True,
            "asd.com?hello=https%3A//www.googleapis.com/auth/youtube",
        ),
        (
            {
                "hello": AuthScope.MANAGE,
            },
            False,
            False,
            False,
            "asd.com?hello=AuthScope.MANAGE",
        ),
    ],
    ids=[
        "None value removed",
        "None value",
        "Split list",
        "Non split list",
        "Enum value",
        "Non enum value",
    ],
)
async def test_build_url(
    params: dict[str, Any],
    remove_none: bool,
    split_lists: bool,
    enum_value: bool,
    result: str,
) -> None:
    """Test build url."""
    assert build_url("asd.com", params, remove_none, split_lists, enum_value) == result



"""Tests for the YouTube client."""
import json
from datetime import datetime, timezone

import aiohttp
import pytest
from aresponses import ResponsesMockServer

from youtubeaio.models import YouTubeChannelThumbnails
from youtubeaio.types import PartMissingError
from youtubeaio.youtube import YouTube

from . import construct_fixture, load_fixture
from .const import YOUTUBE_URL
from .helper import get_thumbnail


async def test_fetch_channel(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving a channel."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("channel_response_snippet.json"),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        channel_generator = youtube.get_channels(
            channel_ids=["UC_x5XG1OV2P6uZZ5FSM9Ttw"],
        )
        channel = await channel_generator.__anext__()
        assert channel
        assert channel.channel_id == "UC_x5XG1OV2P6uZZ5FSM9Ttw"
        assert channel.upload_playlist_id == "UU_x5XG1OV2P6uZZ5FSM9Ttw"
        assert channel.snippet
        assert channel.snippet.published_at == datetime(
            2007,
            8,
            23,
            0,
            34,
            43,
            tzinfo=timezone.utc,
        )
        with pytest.raises(StopAsyncIteration):
            await channel_generator.__anext__()
        await youtube.close()


async def test_fetch_own_channel(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving own channel."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/channels?part=snippet&mine=true",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("channel_response_snippet.json"),
        ),
        match_querystring=True,
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        channel_generator = youtube.get_user_channels()
        channel = await channel_generator.__anext__()
        assert channel
        assert channel.snippet
        assert channel.snippet.published_at == datetime(
            2007,
            8,
            23,
            0,
            34,
            43,
            tzinfo=timezone.utc,
        )
        with pytest.raises(StopAsyncIteration):
            await channel_generator.__anext__()
        await youtube.close()


@pytest.mark.parametrize(
    ("thumbnails", "result_url"),
    [
        (
            YouTubeChannelThumbnails(
                high=get_thumbnail("high"),
                medium=get_thumbnail("medium"),
                default=get_thumbnail("default"),
            ),
            "high",
        ),
        (
            YouTubeChannelThumbnails(
                high=None,
                medium=get_thumbnail("medium"),
                default=get_thumbnail("default"),
            ),
            "medium",
        ),
        (
            YouTubeChannelThumbnails(
                high=None,
                medium=None,
                default=get_thumbnail("default"),
            ),
            "default",
        ),
    ],
)
async def test_get_hq_thumbnail(
    thumbnails: YouTubeChannelThumbnails,
    result_url: str,
) -> None:
    """Check if the highest quality thumbnail is returned."""
    assert thumbnails.get_highest_quality().url == result_url


async def test_nullable_fields(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if the fields exist when they are filled."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(
                construct_fixture(
                    "channel",
                    ["snippet", "contentDetails", "statistics"],
                    1,
                ),
            ),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        async for subscription in youtube.get_user_channels():
            assert subscription
            assert subscription.snippet
            assert subscription.content_details
            assert subscription.statistics


async def test_nullable_fields_null(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if an error is thrown if a non-requested part is accessed."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(construct_fixture("channel", [], 1)),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        async for subscription in youtube.get_user_channels():
            assert subscription
            with pytest.raises(PartMissingError):
                assert subscription.snippet
            with pytest.raises(PartMissingError):
                assert subscription.content_details
            with pytest.raises(PartMissingError):
                assert subscription.statistics



"""Tests for the YouTube client."""
import json
from datetime import datetime, timezone

import aiohttp
import pytest
from aiohttp.web_request import BaseRequest
from aresponses import Response, ResponsesMockServer

from youtubeaio.helper import first
from youtubeaio.models import YouTubeVideoThumbnails
from youtubeaio.types import PartMissingError
from youtubeaio.youtube import YouTube

from . import construct_fixture, load_fixture
from .const import YOUTUBE_URL
from .helper import get_thumbnail


async def test_fetch_video(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving a video."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("video_response_snippet.json"),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        video = await youtube.get_video(video_id="Ks-_Mh1QhMc")
        assert video
        assert video.snippet
        assert video.snippet.published_at == datetime(
            2012,
            10,
            1,
            15,
            27,
            35,
            tzinfo=timezone.utc,
        )
        assert video.snippet.channel_id == "UCAuUUnT6oDeKwE6v1NGQxug"
        assert (
            video.snippet.title
            == "Your body language may shape who you are | Amy Cuddy"
        )
        assert (
            video.snippet.description
            == "Body language affects how others see us, but it may also change how "
            'we see ourselves. Social psychologist Amy Cuddy argues that "power '
            "posing\" -- standing in a posture of confidence, even when we don't "
            "feel confident -- can boost feelings of confidence, and might have an "
            "impact on our chances for success. (Note: Some of the findings "
            "presented in this talk have been referenced in an ongoing debate "
            "among social scientists about robustness and reproducibility. Read "
            "Amy Cuddy's response here: "
            "http://ideas.ted.com/inside-the-debate-about-power-posing-a-q-a-with"
            "-amy-cuddy/)\n\nGet TED Talks recommended just for you! Learn more at "
            "https://www.ted.com/signup.\n\nThe TED Talks channel features the "
            "best talks and performances from the TED Conference, where the "
            "world's leading thinkers and doers give the talk of their lives in 18 "
            "minutes (or less). Look for talks on Technology, Entertainment and "
            "Design -- plus science, business, global issues, the arts and "
            "more.\n\nFollow TED on Twitter: http://www.twitter.com/TEDTalks\nLike "
            "TED on Facebook: https://www.facebook.com/TED\n\nSubscribe to our "
            "channel: https://www.youtube.com/TED"
        )
        assert video.snippet.thumbnails
        assert (
            video.snippet.thumbnails.default.url
            == "https://i.ytimg.com/vi/Ks-_Mh1QhMc/default.jpg"
        )
        assert video.snippet.thumbnails.default.width == 120
        assert video.snippet.thumbnails.default.height == 90
        assert video.snippet.thumbnails.medium
        assert (
            video.snippet.thumbnails.medium.url
            == "https://i.ytimg.com/vi/Ks-_Mh1QhMc/mqdefault.jpg"
        )
        assert video.snippet.thumbnails.medium.width == 320
        assert video.snippet.thumbnails.medium.height == 180
        assert video.snippet.thumbnails.high
        assert (
            video.snippet.thumbnails.high.url
            == "https://i.ytimg.com/vi/Ks-_Mh1QhMc/hqdefault.jpg"
        )
        assert video.snippet.thumbnails.high.width == 480
        assert video.snippet.thumbnails.high.height == 360
        assert video.snippet.thumbnails.standard
        assert (
            video.snippet.thumbnails.standard.url
            == "https://i.ytimg.com/vi/Ks-_Mh1QhMc/sddefault.jpg"
        )
        assert video.snippet.thumbnails.standard.width == 640
        assert video.snippet.thumbnails.standard.height == 480
        assert video.snippet.thumbnails.maxres
        assert (
            video.snippet.thumbnails.maxres.url
            == "https://i.ytimg.com/vi/Ks-_Mh1QhMc/maxresdefault.jpg"
        )
        assert video.snippet.thumbnails.maxres.width == 1280
        assert video.snippet.thumbnails.maxres.height == 720
        await youtube.close()


async def test_fetch_videos(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving a list of videos."""

    async def response_handler(req: BaseRequest) -> Response:
        """Response handler for this test."""
        if "pageToken" in req.query:
            fixture = "video_response_2.json"
        else:
            fixture = "video_response_1.json"
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture(fixture),
        )

    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        response_handler,
        repeat=2,
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        videos = youtube.get_videos(
            video_ids=["Ks-_Mh1QhMc", "GvgqDSnpRQM", "V4DDt30Aat4"],
        )
        video1 = await videos.__anext__()
        assert video1
        assert video1.video_id == "Ks-_Mh1QhMc"
        video2 = await videos.__anext__()
        assert video2
        assert video2.video_id == "GvgqDSnpRQM"
        video3 = await videos.__anext__()
        assert video3
        assert video3.video_id == "V4DDt30Aat4"


async def test_fetch_single_page_video(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving a page of videos."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("video_response_2.json"),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        videos = youtube.get_videos(
            video_ids=["V4DDt30Aat4"],
        )
        video3 = await videos.__anext__()
        assert video3
        assert video3.video_id == "V4DDt30Aat4"
        with pytest.raises(StopAsyncIteration):
            await videos.__anext__()


async def test_fetch_no_videos() -> None:
    """Test retrieving no videos."""
    youtube = YouTube()
    with pytest.raises(ValueError):
        await first(youtube.get_videos(video_ids=[]))


@pytest.mark.parametrize(
    ("thumbnails", "result_url"),
    [
        (
            YouTubeVideoThumbnails(
                maxres=get_thumbnail("maxres"),
                standard=get_thumbnail("standard"),
                high=get_thumbnail("high"),
                medium=get_thumbnail("medium"),
                default=get_thumbnail("default"),
            ),
            "maxres",
        ),
        (
            YouTubeVideoThumbnails(
                maxres=None,
                standard=get_thumbnail("standard"),
                high=get_thumbnail("high"),
                medium=get_thumbnail("medium"),
                default=get_thumbnail("default"),
            ),
            "standard",
        ),
        (
            YouTubeVideoThumbnails(
                maxres=None,
                standard=None,
                high=get_thumbnail("high"),
                medium=get_thumbnail("medium"),
                default=get_thumbnail("default"),
            ),
            "high",
        ),
        (
            YouTubeVideoThumbnails(
                maxres=None,
                standard=None,
                high=None,
                medium=get_thumbnail("medium"),
                default=get_thumbnail("default"),
            ),
            "medium",
        ),
        (
            YouTubeVideoThumbnails(
                maxres=None,
                standard=None,
                high=None,
                medium=None,
                default=get_thumbnail("default"),
            ),
            "default",
        ),
    ],
)
async def test_get_hq_thumbnail(
    thumbnails: YouTubeVideoThumbnails,
    result_url: str,
) -> None:
    """Check if the highest quality thumbnail is returned."""
    assert thumbnails.get_highest_quality().url == result_url


async def test_nullable_fields(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if the fields exist when they are filled."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(construct_fixture("video", ["snippet"], 1)),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        video = await youtube.get_video(video_id="V4DDt30Aat4")
        assert video
        assert video.snippet.channel_id == "UCAuUUnT6oDeKwE6v1NGQxug"


async def test_nullable_fields_null(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if an error is thrown if a non-requested part is accessed."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/videos",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(construct_fixture("video", [], 1)),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        video = await youtube.get_video(video_id="V4DDt30Aat4")
        assert video
        with pytest.raises(PartMissingError):
            assert video.snippet.thumbnails



"""Tests for the YouTube client."""
import json

import aiohttp
import pytest
from aresponses import ResponsesMockServer

from youtubeaio.types import PartMissingError
from youtubeaio.youtube import YouTube

from . import construct_fixture, load_fixture
from .const import YOUTUBE_URL


async def test_fetch_playlist_items(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving playlist items."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/playlistItems",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("playlist_item_response_snippet_content_details.json"),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        count = 0
        async for playlist_item in youtube.get_playlist_items(
            "UU_x5XG1OV2P6uZZ5FSM9Ttw",
        ):
            count += 1
            assert playlist_item
            assert playlist_item.snippet
            assert playlist_item.content_details
        assert count == 5
        await youtube.close()


async def test_nullable_fields(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if the fields exist when they are filled."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/playlistItems",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(
                construct_fixture("playlist_item", ["snippet", "contentDetails"], 1),
            ),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        async for playlist_item in youtube.get_playlist_items(
            "UU_x5XG1OV2P6uZZ5FSM9Ttw",
        ):
            assert playlist_item
            assert playlist_item.snippet
            assert playlist_item.content_details


async def test_nullable_fields_null(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if an error is thrown if a non-requested part is accessed."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/playlistItems",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(construct_fixture("playlist_item", [], 1)),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        async for playlist_item in youtube.get_playlist_items(
            "UU_x5XG1OV2P6uZZ5FSM9Ttw",
        ):
            assert playlist_item
            with pytest.raises(PartMissingError):
                assert playlist_item.snippet
            with pytest.raises(PartMissingError):
                assert playlist_item.content_details



"""Tests for the YouTube client."""

import aiohttp
import pytest
from aiohttp.web_request import BaseRequest
from aresponses import Response, ResponsesMockServer

from youtubeaio.oauth import refresh_access_token
from youtubeaio.types import InvalidRefreshTokenError, UnauthorizedError

from . import load_fixture


async def test_refresh_access_token(
    aresponses: ResponsesMockServer,
) -> None:
    """Test setting user authentication."""

    async def response_handler(request: BaseRequest) -> Response:
        """Response handler for this test."""
        assert (
            await request.text()
            == "refresh_token=asdasd&client_id=app_id&grant_type=refresh_token&"
            "client_secret=app_secret&access_type=offline&prompt=consent"
        )
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("refresh_token.json"),
        )

    aresponses.add(
        "oauth2.googleapis.com",
        "/token",
        "POST",
        response_handler,
    )
    async with aiohttp.ClientSession() as session:
        await refresh_access_token("asdasd", "app_id", "app_secret", session)


async def test_refresh_access_token_new_session(
    aresponses: ResponsesMockServer,
) -> None:
    """Test setting user authentication with new session."""
    aresponses.add(
        "oauth2.googleapis.com",
        "/token",
        "POST",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("refresh_token.json"),
        ),
    )
    await refresh_access_token("asdasd", "app_id", "app_secret")


async def test_refresh_access_token_invalid_token(
    aresponses: ResponsesMockServer,
) -> None:
    """Test setting user authentication with invalid refresh token."""
    aresponses.add(
        "oauth2.googleapis.com",
        "/token",
        "POST",
        aresponses.Response(
            status=400,
            headers={"Content-Type": "application/json"},
            text='{"error": "Invalid token"}',
        ),
    )
    with pytest.raises(InvalidRefreshTokenError):
        await refresh_access_token("asdasd", "app_id", "app_secret")


async def test_refresh_access_token_unauthorized(
    aresponses: ResponsesMockServer,
) -> None:
    """Test setting user authentication while unauthorized."""
    aresponses.add(
        "oauth2.googleapis.com",
        "/token",
        "POST",
        aresponses.Response(
            status=401,
            headers={"Content-Type": "application/json"},
            text='{"error": "Unauthorized"}',
        ),
    )
    with pytest.raises(UnauthorizedError):
        await refresh_access_token("asdasd", "app_id", "app_secret")



"""Tests for the YouTube client."""

import aiohttp
import pytest

from youtubeaio.types import AuthScope, MissingScopeError
from youtubeaio.youtube import YouTube


async def test_user_authentication() -> None:
    """Test setting user authentication."""
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        await youtube.set_user_authentication("token", [AuthScope.READ_ONLY], "refresh")
        assert youtube.get_user_auth_token() == "token"
        await youtube.close()


async def test_user_authentication_without_scopes() -> None:
    """Test setting user authentication without scopes."""
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        with pytest.raises(MissingScopeError):
            await youtube.set_user_authentication("token", [], "refresh")
        await youtube.close()


async def test_user_authentication_without_refresh_token() -> None:
    """Test setting user authentication without refresh token."""
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session, app_id="asd", app_secret="asd")
        with pytest.raises(ValueError):
            await youtube.set_user_authentication("token", [AuthScope.READ_ONLY])
        youtube = YouTube(session=session, auto_refresh_auth=False)
        await youtube.set_user_authentication("token", [AuthScope.READ_ONLY])
        assert youtube.get_user_auth_token() == "token"
        await youtube.close()



"""Tests for the YouTube client."""
import json

import aiohttp
import pytest
from aresponses import ResponsesMockServer

from youtubeaio.types import PartMissingError
from youtubeaio.youtube import YouTube

from . import construct_fixture, load_fixture
from .const import YOUTUBE_URL


async def test_fetch_user_subscriptions(
    aresponses: ResponsesMockServer,
) -> None:
    """Test retrieving a video."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/subscriptions?part=snippet&mine=true",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("subscription_response_snippet.json"),
        ),
        match_querystring=True,
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        count = 0
        async for subscription in youtube.get_user_subscriptions():
            count += 1
            assert subscription
            assert subscription.snippet
            assert subscription.snippet.channel_id
        assert count == 2
        await youtube.close()


async def test_nullable_fields(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if the fields exist when they are filled."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/subscriptions",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(construct_fixture("subscription", ["snippet"], 1)),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        async for subscription in youtube.get_user_subscriptions():
            assert subscription
            assert subscription.snippet


async def test_nullable_fields_null(
    aresponses: ResponsesMockServer,
) -> None:
    """Check if an error is thrown if a non-requested part is accessed."""
    aresponses.add(
        YOUTUBE_URL,
        "/youtube/v3/subscriptions",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=json.dumps(construct_fixture("subscription", [], 1)),
        ),
    )
    async with aiohttp.ClientSession() as session:
        youtube = YouTube(session=session)
        async for subscription in youtube.get_user_subscriptions():
            assert subscription
            with pytest.raises(PartMissingError):
                assert subscription.snippet.channel_id
